/**
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.rempl.api;

// for IO manipulations
import java.io.IOException;
import java.io.Reader;
import java.io.Writer;

// for collections
import java.util.Collection;

/**
 * MOF 2.0 model abstraction.
 *
 * <p>The model is a holder of instances
 * of MOF classes. Every instance is called an {@link Element} and
 * has name and a number of slots. The model can accept transformations
 * via {@link #transform(String)} and respond to OCL queries via
 * {@link #query(String)}. Internal implementation of the model has
 * to be able to parse transformation requests and queries.
 *
 * <p>Transformation request is a declaration of MOF instances, in
 * the following simple notation:
 *
 * <pre>
 * {@code
 * :Class {
 *   name: "Employee";
 *   owner: dept;
 *   visibility: public;
 * };
 * dept:Class {
 *   name: "Department";
 *   visibility: public;
 * };
 * }
 * </pre>
 *
 * <p>The example creates two instances of MOF meta-class <tt>"Class"</tt>,
 * where one of them is named (<tt>"dept:Class"</tt>) and another one
 * is anonymous (<tt>":Class"</tt>). The model has to accept such
 * request and transform its internal state according to it. In most
 * cases the model has to create new MOF elements. But it's totally up
 * to its own discret. Some transformation requests may be ignored.
 *
 * <p>Query request is an attempt to find and retrieve instances of
 * certain meta-class that satisfy the supplied OCL invariant. For
 * example:
 *
 * <pre>
 * {@code
 * context Class
 * inv: name = "Employee"
 * }
 * </pre>
 *
 * <p>This request will find and return all instances of meta-class
 * <tt>"Class"</tt> where slot <tt>"name"</tt> contains value
 * <tt>"Employee"</tt>. Ultimately the model has to support all possible
 * OCL 2.2 expressions.
 *
 * <p>Both transformation and query requests work on UML 2.0 meta-model,
 * meaning that meta-classes and relations between them are defined
 * in the UML 2.0 specification.
 *
 * <p>By means of {@link #save(Writer)} and {@link #load(Reader)} methods
 * the model has to be able to save and restore itself in XMI format.
 *
 * @author Yegor Bugayenko (yegor@rempl.com)
 * @version $Id: Model.java 1239 2011-04-20 18:21:29Z guard $
 * @see <a href="http://www.omg.org/spec/OCL/2.2/">OCL 2.2</a>
 * @see <a href="http://www.omg.org/spec/UML/2.0/">UML 2.0</a>
 * @see <a href="http://www.omg.org/spec/XMI/">XMI</a>
 * @todo #24 For saving and loading of the model we should use a special
 *       class that will decouple real writing/reading IO operations
 *       from the REMPL API. Moreover, we will be able to introduce
 *       other mechanisms of model serialization, not only through XMI,
 *       but into a database directly, for example. This approach
 *       will help REMPL-OSS to catch model saving operation and
 *       inject data directly into SQL database.
 */
public interface Model {

    /**
     * Prefix to be used in values in order to denote
     * that they are references to other objects.
     * @todo #2 This mechanism is temporary and ugly. It has to
     *       be replaced with a better approach. We should
     *       understand links right inside the #transform() method
     *       without any prefixes.
     */
    String REF_PREFIX = "^";

    /**
     * Request a transformation providing a snippet of new
     * specification. The tranformation has to be done inside
     * the model implementation, while some parts of it (or
     * the entire request) may be ignored.
     * @param query The transformation
     */
    void transform(final String query);

    /**
     * Request resolution of an OCL invariant. The model has to
     * find all possible instances of "contexted" class and
     * return a collection of them.
     * @param query The OCL invariant
     * @return The collection of found elements
     * @see Element
     * @see <a href="http://www.omg.org/spec/OCL/2.2/">OCL 2.2</a>
     */
    Collection<Element> query(final String query);

    /**
     * Save the model to the {@link Writer}. The model has to save its
     * internal state in XMI format to the writer.
     * @param writer The writer accepting XMI
     * @throws IOException If some I/O problem occurs
     * @see <a href="http://www.omg.org/spec/XMI/">XMI</a>
     */
    void save(final Writer writer) throws IOException;

    /**
     * Load it from the {@link Reader}.
     * @param reader The reader with XMI inside
     * @throws IOException If the provided XMI is invalid for some reason
     *                     or some other I/O problem occurs
     * @see <a href="http://www.omg.org/spec/XMI/">XMI</a>
     */
    void load(final Reader reader) throws IOException;

}
