/**
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.rempl.api;

// for collections processing
import java.util.ArrayList;
import java.util.List;

// global XML constants
import javax.xml.XMLConstants;

// DOM Source, transformations
import javax.xml.transform.dom.DOMSource;

// XML Schema validation
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;

// string and file processing
import org.apache.commons.lang.StringUtils;

// DOM
import org.w3c.dom.Document;

// SAX
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * Validator of XML.
 *
 * @author Yegor Bugayenko (yegor@rempl.com)
 * @version $Id: ROMValidator.java 668 2011-03-14 22:35:32Z yegor256@yahoo.com $
 * @see ROMUtils#parse(String)
 */
final class ROMValidator {

    /**
     * XML schema factory, DOM.
     */
    private static SchemaFactory schemaFactory =
        SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);

    /**
     * Validate the document.
     * @param document The Document created
     * @throws ROMException If some ROM parsing problem occurs
     */
    public void validate(final Document document) throws ROMException {
        final StrictErrorHandler handler =
            new ROMValidator.StrictErrorHandler();
        try {
            final Validator validator = ROMValidator.schemaFactory
                .newSchema(this.getClass().getResource("/rom.xsd"))
                .newValidator();
            validator.setErrorHandler(handler);
            validator.validate(new DOMSource(document));
        } catch (org.xml.sax.SAXException ex) {
            throw new ROMException(ex);
        } catch (java.io.IOException ex) {
            throw new ROMException(ex);
        }
        if (!handler.isEmpty()) {
            throw new ROMException(handler.toString());
        }
    }

    /**
     * XML Schema validation error handler. It is much stricter than
     * the default one, and we are using it in {@link #validate(Document)}.
     * It is public because of unit-tests.
     * @see #validate(Document)
     */
    public static final class StrictErrorHandler implements ErrorHandler {
        /**
         * Collection of reported problems.
         */
        private List<String> errors = new ArrayList<String>();
        /**
         * {@inheritDoc}
         * @param exn Exception thrown
         * @throws SAXException If it's really fatal
         */
        @Override
        public void warning(final SAXParseException exn)
            throws SAXException {
            this.errors.add(exn.getMessage());
        }
        /**
         * {@inheritDoc}
         * @param exn Exception thrown
         * @throws SAXException If it's really fatal
         */
        @Override
        public void error(final SAXParseException exn)
            throws SAXException {
            this.errors.add(exn.getMessage());
        }
        /**
         * {@inheritDoc}
         * @param exn Exception thrown
         * @throws SAXException If it's really fatal
         */
        @Override
        public void fatalError(final SAXParseException exn)
            throws SAXException {
            this.errors.add(exn.getMessage());
        }
        /**
         * {@inheritDoc}
         */
        @Override
        public String toString() {
            return StringUtils.join(this.errors, "; ");
        }
        /**
         * Is it empty?
         * @return Is it?
         */
        public boolean isEmpty() {
            return this.errors.isEmpty();
        }
    }

}
