/**
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.rempl.api.uml;

// for manipulation with elements
import java.util.ArrayList;
import java.util.List;

/**
 * Builder of UML "diagram". It's a text presenation of UML,
 * used later in {@link com.rempl.api.Model#transform(String)}. You
 * can use the diagram like the following example:
 *
 * <pre>
 * {@code
 * Diagram diagram = new Diagram();
 * Instance file = diagram.add("Artifact");
 * file.add("fileName", "/home/me/file.txt")
 *   .add("isAbstract", false)
 *   .add("size", 1024);
 * String uml = diagram.uml();
 * model.transform(uml);
 * }
 * </pre>
 *
 * @author Yegor Bugayenko (yegor@rempl.com)
 * @version $Id: Diagram.java 1026 2011-03-30 09:38:04Z yegor256@yahoo.com $
 * @see com.rempl.api.Model#transform(String)
 */
public final class Diagram {

    /**
     * Collection of UML instances.
     * @see #add(String)
     */
    private final List<Instance> instances =
        new ArrayList<Instance>();

    /**
     * Create new element and add it to the diagram. The
     * method returns the instance back, and it can be used
     * later for adding associations and attributes to it.
     * @param type The type of the instance to create
     * @return The element just created
     */
    public Instance add(final String type) {
        final Instance instance = new Instance(type);
        this.instances.add(instance);
        return instance;
    }

    /**
     * Convert the diagram to UML. The string produced by this
     * method shall be used later for Model generation/loading
     * in {@link com.rempl.api.Model#transform(String)}.
     * @return The UML
     * @see com.rempl.api.Model#transform(String)
     */
    public String uml() {
        final StringBuffer uml = new StringBuffer();
        for (Instance instance : this.instances) {
            uml.append(instance.uml(this));
        }
        return uml.toString();
    }

    /**
     * Calculate the ID of the instance. Every instance in the
     * diagram has its own unique identifier, which is later used
     * in the UML produced for the {@link com.rempl.api.Model}.
     *
     * <p>The method is <tt>protected</tt> because it is called
     * only from inside this package. You can't call it directly.
     *
     * @param instance The instance
     * @return The UML ID
     * @see Instance#uml(Diagram)
     */
    protected String instanceId(final Instance instance) {
        return "i" + this.instances.indexOf(instance);
    }

}
